@@ -30,6 +30,11 @@
 
 typedef void (*test_hook) (bson_t *test);
 
+typedef struct {
+   const char *description;
+   const char *reason;
+} test_skip_t;
+
 typedef struct _json_test_config_t {
    void *ctx;
    const bson_t *scenario;
@@ -39,12 +44,15 @@ typedef struct _json_test_config_t {
    bool command_started_events_only;
    bool command_monitoring_allow_subset;
    const char *uri_str;
+   /* skips is a NULL terminated list of tests to skip identified by the test
+    * "description" */
+   test_skip_t *skips;
 } json_test_config_t;
 
 
-#define JSON_TEST_CONFIG_INIT                                \
-   {                                                         \
-      NULL, NULL, NULL, NULL, NULL, NULL, false, false, NULL \
+#define JSON_TEST_CONFIG_INIT                                      \
+   {                                                               \
+      NULL, NULL, NULL, NULL, NULL, NULL, false, false, NULL, NULL \
    }
 
 bson_t *
