@@ -75,6 +75,12 @@ char *
 test_framework_get_unix_domain_socket_path_escaped (void);
 mongoc_uri_t *
 test_framework_get_uri (void);
+mongoc_uri_t *
+test_framework_get_uri_multi_mongos_loadbalanced (void);
+bool
+test_framework_uri_apply_multi_mongos (mongoc_uri_t *uri,
+                                       bool use_multi,
+                                       bson_error_t *error);
 size_t
 test_framework_mongos_count (void);
 char *
@@ -94,10 +100,23 @@ void
 test_framework_set_ssl_opts (mongoc_client_t *client);
 void
 test_framework_set_pool_ssl_opts (mongoc_client_pool_t *pool);
+
+mongoc_server_api_t *
+test_framework_get_default_server_api (void);
+
 mongoc_client_t *
-test_framework_client_new (void);
+test_framework_new_default_client (void);
+mongoc_client_t *
+test_framework_client_new (const char *uri_str, const mongoc_server_api_t *api);
+mongoc_client_t *
+test_framework_client_new_from_uri (const mongoc_uri_t *uri,
+                                    const mongoc_server_api_t *api);
+
 mongoc_client_pool_t *
-test_framework_client_pool_new (void);
+test_framework_new_default_client_pool (void);
+mongoc_client_pool_t *
+test_framework_client_pool_new_from_uri (const mongoc_uri_t *uri,
+                                         const mongoc_server_api_t *api);
 
 bool
 test_framework_is_mongos (void);
@@ -117,6 +136,8 @@ test_framework_max_wire_version_at_least (int version);
 int64_t
 test_framework_max_write_batch_size (void);
 
+bool
+test_framework_has_auth (void);
 int
 test_framework_skip_if_auth (void);
 int
@@ -130,6 +151,8 @@ test_framework_skip_if_crypto (void);
 int
 test_framework_skip_if_no_crypto (void);
 int
+test_framework_skip_if_no_mongohouse (void);
+int
 test_framework_skip_if_mongos (void);
 int
 test_framework_skip_if_replset (void);
@@ -169,6 +192,10 @@ WIRE_VERSION_CHECK_DECLS (6)
 WIRE_VERSION_CHECK_DECLS (7)
 WIRE_VERSION_CHECK_DECLS (8)
 WIRE_VERSION_CHECK_DECLS (9)
+/* wire versions 10, 11, 12 were internal to the 5.0 release cycle */
+WIRE_VERSION_CHECK_DECLS (13)
+/* wire version 14 begins with the 5.1 prerelease. */
+WIRE_VERSION_CHECK_DECLS (14)
 
 #undef WIRE_VERSION_CHECK_DECLS
 
@@ -219,4 +246,22 @@ test_framework_skip_if_no_aws (void);
 int
 test_framework_skip_if_no_setenv (void);
 
+bool
+test_framework_supports_legacy_opcodes (void);
+
+int
+test_framework_skip_if_no_legacy_opcodes (void);
+
+int
+test_framework_skip_if_no_getlasterror (void);
+
+int
+test_framework_skip_if_no_exhaust_cursors (void);
+
+bool
+test_framework_is_serverless (void);
+
+bool
+test_framework_is_loadbalanced (void);
+
 #endif
