@@ -41,7 +41,7 @@ typedef struct {
 typedef enum {
    TF_FAST_HEARTBEAT = 1 << 0,
    TF_FAST_MIN_HEARTBEAT = 1 << 1,
-   TF_AUTO_RESPOND_POLLING_ISMASTER = 1 << 2
+   TF_AUTO_RESPOND_POLLING_HELLO = 1 << 2
 } tf_flags_t;
 
 typedef struct {
@@ -187,9 +187,10 @@ _server_changed (const mongoc_apm_server_changed_t *event)
    "{ 'processId': { '$oid': 'AABBAABBAABBAABBAABBAABB' }, 'counter': 1 }"
 
 bool
-auto_respond_polling_ismaster (request_t *request, void *ctx)
+auto_respond_polling_hello (request_t *request, void *ctx)
 {
-   if (0 == strcmp (request->command_name, "isMaster")) {
+   if (0 == strcasecmp (request->command_name, HANDSHAKE_CMD_LEGACY_HELLO) ||
+       0 == strcmp (request->command_name, "hello")) {
       const bson_t *doc;
 
       doc = request_get_doc ((request), 0);
@@ -225,7 +226,8 @@ tf_new (tf_flags_t flags)
    mongoc_apm_set_server_heartbeat_succeeded_cb (callbacks,
                                                  _heartbeat_succeeded);
    mongoc_apm_set_server_heartbeat_failed_cb (callbacks, _heartbeat_failed);
-   tf->pool = mongoc_client_pool_new (mock_server_get_uri (tf->server));
+   tf->pool = test_framework_client_pool_new_from_uri (
+      mock_server_get_uri (tf->server), NULL);
    mongoc_client_pool_set_apm_callbacks (tf->pool, callbacks, tf);
    mongoc_apm_callbacks_destroy (callbacks);
 
@@ -241,9 +243,9 @@ tf_new (tf_flags_t flags)
          ->min_heartbeat_frequency_msec = FAST_HEARTBEAT_MS;
    }
 
-   if (flags & TF_AUTO_RESPOND_POLLING_ISMASTER) {
+   if (flags & TF_AUTO_RESPOND_POLLING_HELLO) {
       mock_server_autoresponds (
-         tf->server, auto_respond_polling_ismaster, NULL, NULL);
+         tf->server, auto_respond_polling_hello, NULL, NULL);
    }
    tf->flags = flags;
    tf->logs = bson_string_new ("");
@@ -362,7 +364,7 @@ test_connect_succeeds (void)
    request_t *request;
 
    tf = tf_new (0);
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 1);
    mock_server_replies_ok_and_destroys (request);
@@ -383,7 +385,7 @@ test_connect_hangup (void)
    request_t *request;
 
    tf = tf_new (0);
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 1);
    OBSERVE (tf, !tf->observations->awaited);
@@ -409,7 +411,7 @@ test_connect_badreply (void)
    request_t *request;
 
    tf = tf_new (0);
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_started == 1);
    mock_server_replies_simple (request, "{'ok': 0}");
@@ -434,7 +436,7 @@ test_connect_shutdown (void)
    request_t *request;
 
    tf = tf_new (0);
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_started == 1);
    /* Before the server replies, signal the server monitor to shutdown. */
@@ -460,7 +462,7 @@ test_connect_requestscan (void)
    request_t *request;
 
    tf = tf_new (0);
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    /* Before the mock server replies, request a scan. */
    _request_scan (tf);
@@ -487,7 +489,7 @@ test_retry_succeeds (void)
    tf = tf_new (TF_FAST_MIN_HEARTBEAT);
 
    /* Initial discovery occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 1);
    mock_server_replies_ok_and_destroys (request);
@@ -500,8 +502,8 @@ test_retry_succeeds (void)
    /* Request a scan to speed things up. */
    _request_scan (tf);
 
-   /* The next ismaster occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   /* The next hello occurs. */
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    mock_server_hangs_up (request);
@@ -513,7 +515,7 @@ test_retry_succeeds (void)
    OBSERVE_SOON (tf, tf->observations->sd_type == MONGOC_SERVER_UNKNOWN);
 
    /* Retry occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 3);
    mock_server_replies_ok_and_destroys (request);
@@ -533,7 +535,7 @@ test_retry_hangup (void)
    tf = tf_new (TF_FAST_MIN_HEARTBEAT);
 
    /* Initial discovery occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 1);
    mock_server_replies_ok_and_destroys (request);
@@ -546,8 +548,8 @@ test_retry_hangup (void)
    /* Request a scan to speed things up. */
    _request_scan (tf);
 
-   /* The next ismaster occurs (due to fast heartbeat). */
-   request = mock_server_receives_ismaster (tf->server);
+   /* The next hello occurs (due to fast heartbeat). */
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    mock_server_hangs_up (request);
@@ -559,7 +561,7 @@ test_retry_hangup (void)
    OBSERVE_SOON (tf, tf->observations->sd_type == MONGOC_SERVER_UNKNOWN);
 
    /* Retry occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 3);
    mock_server_hangs_up (request);
@@ -580,7 +582,7 @@ test_retry_badreply (void)
    tf = tf_new (TF_FAST_MIN_HEARTBEAT);
 
    /* Initial discovery occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 1);
    mock_server_replies_ok_and_destroys (request);
@@ -593,8 +595,8 @@ test_retry_badreply (void)
    /* Request a scan to speed things up. */
    _request_scan (tf);
 
-   /* The next ismaster occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   /* The next hello occurs. */
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    mock_server_hangs_up (request);
@@ -606,7 +608,7 @@ test_retry_badreply (void)
    OBSERVE_SOON (tf, tf->observations->sd_type == MONGOC_SERVER_UNKNOWN);
 
    /* Retry occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 3);
    mock_server_replies_simple (request, "{'ok': 0}");
@@ -628,7 +630,7 @@ test_retry_shutdown (void)
    tf = tf_new (TF_FAST_HEARTBEAT);
 
    /* Initial discovery occurs. */
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 1);
    mock_server_replies_ok_and_destroys (request);
@@ -638,8 +640,8 @@ test_retry_shutdown (void)
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_failed == 0);
    OBSERVE_SOON (tf, tf->observations->sd_type == MONGOC_SERVER_STANDALONE);
 
-   /* The next ismaster occurs (due to fast heartbeat). */
-   request = mock_server_receives_ismaster (tf->server);
+   /* The next hello occurs (due to fast heartbeat). */
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    _signal_shutdown (tf);
@@ -666,7 +668,7 @@ test_flip_flop (void)
    tf = tf_new (0);
 
    for (i = 1; i < 100; i++) {
-      request = mock_server_receives_ismaster (tf->server);
+      request = mock_server_receives_legacy_hello (tf->server, NULL);
       OBSERVE (tf, request);
       mock_server_replies_ok_and_destroys (request);
       _signal_shutdown (tf);
@@ -685,13 +687,13 @@ test_repeated_requestscan (void)
    request_t *request;
    int i;
 
-   /* Multiple repeated requests before an ismaster completes should not cause a
+   /* Multiple repeated requests before a hello completes should not cause a
     * subsequent scan. */
    tf = tf_new (TF_FAST_MIN_HEARTBEAT);
    for (i = 0; i < 10; i++) {
       _request_scan (tf);
    }
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, request);
    for (i = 0; i < 10; i++) {
       _request_scan (tf);
@@ -713,7 +715,7 @@ test_sleep_after_scan (void)
    /* After handling a scan request */
    tf = tf_new (TF_FAST_MIN_HEARTBEAT);
    _request_scan (tf);
-   request = mock_server_receives_ismaster (tf->server);
+   request = mock_server_receives_legacy_hello (tf->server, NULL);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 1);
    mock_server_replies_ok_and_destroys (request);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_succeeded == 1);
@@ -729,11 +731,11 @@ test_streaming_succeeds (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    OBSERVE (tf, tf->observations->awaited);
@@ -754,11 +756,11 @@ test_streaming_hangup (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    OBSERVE (tf, tf->observations->awaited);
@@ -782,11 +784,11 @@ test_streaming_badreply (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    mock_server_replies_simple (request, "{'ok': 0}");
@@ -797,9 +799,9 @@ test_streaming_badreply (void)
    OBSERVE_SOON (tf, tf->observations->n_server_changed == 2);
    OBSERVE_SOON (tf, tf->observations->sd_type == MONGOC_SERVER_UNKNOWN);
 
-   /* Request an immediate scan to trigger the next polling ismaster. */
+   /* Request an immediate scan to trigger the next polling hello. */
    _request_scan (tf);
-   /* The auto responder will handle the polling ismaster. */
+   /* The auto responder will handle the polling hello. */
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_succeeded == 2);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_failed == 1);
    OBSERVE_SOON (tf, tf->observations->n_server_changed == 3);
@@ -814,17 +816,17 @@ test_streaming_shutdown (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER | TF_FAST_HEARTBEAT);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO | TF_FAST_HEARTBEAT);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    _signal_shutdown (tf);
-   /* This should cancel the ismaster immediately. */
+   /* This should cancel the hello immediately. */
    WAIT_TWO_MIN_HEARTBEAT_MS;
-   /* No further ismaster commands should be sent. */
+   /* No further hello commands should be sent. */
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    request_destroy (request);
    tf_destroy (tf);
@@ -836,16 +838,16 @@ test_streaming_cancel (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    _request_cancel (tf);
 
-   /* This should cancel the ismaster immediately. */
+   /* This should cancel the hello immediately. */
    request_destroy (request);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_succeeded == 1);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_failed == 1);
@@ -860,7 +862,7 @@ test_streaming_cancel (void)
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 4);
    mock_server_replies_ok_and_destroys (request);
@@ -876,11 +878,11 @@ test_moretocome_succeeds (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    OBSERVE (tf, tf->observations->awaited);
@@ -910,11 +912,11 @@ test_moretocome_succeeds (void)
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_succeeded == 4);
    OBSERVE (tf, tf->observations->awaited);
    request_destroy (request);
-   /* Server monitor immediately sends awaitable ismaster. */
+   /* Server monitor immediately sends awaitable hello. */
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_started == 5);
    OBSERVE_SOON (tf, tf->observations->n_heartbeat_failed == 0);
@@ -932,11 +934,11 @@ test_moretocome_hangup (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    OBSERVE (tf, tf->observations->awaited);
@@ -971,11 +973,11 @@ test_moretocome_badreply (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    mock_server_replies_opmsg (
@@ -1011,11 +1013,11 @@ test_moretocome_shutdown (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER | TF_FAST_HEARTBEAT);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO | TF_FAST_HEARTBEAT);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    mock_server_replies_opmsg (
@@ -1046,11 +1048,11 @@ test_moretocome_cancel (void)
    test_fixture_t *tf;
    request_t *request;
 
-   tf = tf_new (TF_AUTO_RESPOND_POLLING_ISMASTER);
+   tf = tf_new (TF_AUTO_RESPOND_POLLING_HELLO);
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 2);
    mock_server_replies_opmsg (
@@ -1085,11 +1087,11 @@ test_moretocome_cancel (void)
    OBSERVE_SOON (tf, tf->observations->n_server_changed == 1);
    OBSERVE_SOON (tf, tf->observations->sd_type == MONGOC_SERVER_STANDALONE);
 
-   /* Server monitor sends a fresh awaitable ismaster. */
+   /* Server monitor sends a fresh awaitable hello. */
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 5);
    mock_server_replies_opmsg (
@@ -1107,7 +1109,7 @@ test_moretocome_cancel (void)
    request = mock_server_receives_msg (
       tf->server,
       MONGOC_MSG_EXHAUST_ALLOWED,
-      tmp_bson ("{'isMaster': 1, 'topologyVersion': { '$exists': true}}"));
+      tmp_bson ("{'topologyVersion': { '$exists': true}}"));
    OBSERVE (tf, request);
    OBSERVE (tf, tf->observations->n_heartbeat_started == 6);
    mock_server_replies_opmsg (
