@@ -165,6 +165,31 @@ _host_list_matches (const bson_t *test, context_t *ctx)
    return ret;
 }
 
+typedef struct {
+   const char *uri_str;
+   const char *reason;
+} skipped_dns_test_t;
+
+skipped_dns_test_t SKIPPED_DNS_TESTS[] = {
+   {"mongodb+srv://test5.test.build.10gen.cc/?authSource=otherDB",
+    "C driver requires username present if any auth fields are present"},
+   {0}};
+
+static bool
+is_test_skipped (const char *uri_str)
+{
+   skipped_dns_test_t *skip;
+
+   for (skip = SKIPPED_DNS_TESTS; skip->uri_str != NULL; skip++) {
+      if (!strcmp (skip->uri_str, uri_str)) {
+         MONGOC_DEBUG (
+            "Skipping test of URI: %s Reason: %s", skip->uri_str, skip->reason);
+         return true;
+      }
+   }
+
+   return false;
+}
 
 static void
 _test_dns_maybe_pooled (bson_t *test, bool pooled)
@@ -182,6 +207,7 @@ _test_dns_maybe_pooled (bson_t *test, bool pooled)
    int n_hosts;
    bson_error_t error;
    bool r;
+   const char *uri_str;
 
    if (!test_framework_get_ssl ()) {
       fprintf (stderr,
@@ -190,12 +216,17 @@ _test_dns_maybe_pooled (bson_t *test, bool pooled)
       abort ();
    }
 
+   uri_str = bson_lookup_utf8 (test, "uri");
+   if (is_test_skipped (uri_str)) {
+      return;
+   }
+
    bson_mutex_init (&ctx.mutex);
    ctx.hosts = NULL;
-   expect_ssl = strstr (bson_lookup_utf8 (test, "uri"), "ssl=false") == NULL;
+   expect_ssl = strstr (uri_str, "ssl=false") == NULL;
    expect_error = _mongoc_lookup_bool (test, "error", false /* default */);
 
-   uri = mongoc_uri_new_with_error (bson_lookup_utf8 (test, "uri"), &error);
+   uri = mongoc_uri_new_with_error (uri_str, &error);
    if (!expect_error) {
       ASSERT_OR_PRINT (uri, error);
    }
@@ -249,19 +280,37 @@ _test_dns_maybe_pooled (bson_t *test, bool pooled)
 
    n_hosts = hosts_count (test);
 
-   if (n_hosts && !expect_error) {
-      r = mongoc_client_command_simple (
-         client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
-      ASSERT_OR_PRINT (r, error);
-      WAIT_UNTIL (_host_list_matches (test, &ctx));
-   } else {
-      r = mongoc_client_command_simple (
-         client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
-      BSON_ASSERT (!r);
-      ASSERT_ERROR_CONTAINS (error,
-                             MONGOC_ERROR_SERVER_SELECTION,
-                             MONGOC_ERROR_SERVER_SELECTION_FAILURE,
-                             "");
+   if (pooled) {
+      if (n_hosts && !expect_error) {
+         WAIT_UNTIL (_host_list_matches (test, &ctx));
+      } else {
+         r = mongoc_client_command_simple (
+            client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
+         BSON_ASSERT (!r);
+         ASSERT_ERROR_CONTAINS (error,
+                                MONGOC_ERROR_SERVER_SELECTION,
+                                MONGOC_ERROR_SERVER_SELECTION_FAILURE,
+                                "");
+      }
+   } else if (NULL == mongoc_uri_get_username (uri)) {
+      /* Skip single-threaded tests containing auth credentials. Monitoring
+       * connections need to authenticate, and the credentials in the tests do
+       * not correspond to the test users. TODO (CDRIVER-4046): unskip these
+       * tests. */
+      if (n_hosts && !expect_error) {
+         r = mongoc_client_command_simple (
+            client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
+         ASSERT_OR_PRINT (r, error);
+         WAIT_UNTIL (_host_list_matches (test, &ctx));
+      } else {
+         r = mongoc_client_command_simple (
+            client, "admin", tmp_bson ("{'ping': 1}"), NULL, NULL, &error);
+         BSON_ASSERT (!r);
+         ASSERT_ERROR_CONTAINS (error,
+                                MONGOC_ERROR_SERVER_SELECTION,
+                                MONGOC_ERROR_SERVER_SELECTION_FAILURE,
+                                "");
+      }
    }
 
    /* the client's URI is updated after initial seedlist discovery (though for
@@ -304,11 +353,17 @@ test_dns (bson_t *test)
 
 
 static int
-test_dns_check (void)
+test_dns_check_replset (void)
 {
    return test_framework_getenv_bool ("MONGOC_TEST_DNS") ? 1 : 0;
 }
 
+static int
+test_dns_check_loadbalanced (void)
+{
+   return test_framework_getenv_bool ("MONGOC_TEST_DNS_LOADBALANCED") ? 1 : 0;
+}
+
 
 /* ensure mongoc_topology_select_server_id handles a NULL error pointer in the
  * code path it follows when the topology scanner is invalid */
@@ -340,19 +395,20 @@ test_all_spec_tests (TestSuite *suite)
 {
    char resolved[PATH_MAX];
 
-   test_framework_resolve_path (JSON_DIR "/initial_dns_seedlist_discovery",
-                                resolved);
+   test_framework_resolve_path (
+      JSON_DIR "/initial_dns_seedlist_discovery/replica-set", resolved);
    install_json_test_suite_with_check (suite,
                                        resolved,
                                        test_dns,
-                                       test_dns_check,
+                                       test_dns_check_replset,
                                        test_framework_skip_if_no_crypto);
 
-   test_framework_resolve_path (JSON_DIR "/initial_dns_auth", resolved);
+   test_framework_resolve_path (
+      JSON_DIR "/initial_dns_seedlist_discovery/load-balanced", resolved);
    install_json_test_suite_with_check (suite,
                                        resolved,
                                        test_dns,
-                                       test_dns_check,
+                                       test_dns_check_loadbalanced,
                                        test_framework_skip_if_no_crypto);
 }
 
@@ -384,6 +440,31 @@ make_hosts (char *first_host, ...)
 
 #define MAKE_HOSTS(...) make_hosts (__VA_ARGS__, NULL)
 
+static void
+dump_hosts (mongoc_host_list_t *hosts)
+{
+   mongoc_host_list_t *host;
+
+   MONGOC_DEBUG ("hosts:");
+   for (host = hosts; host; host = hosts->next) {
+      MONGOC_DEBUG ("- %s", host->host_and_port);
+   }
+}
+
+static void
+dump_topology_description (mongoc_topology_description_t *td)
+{
+   size_t i;
+   mongoc_server_description_t *sd;
+
+   MONGOC_DEBUG ("topology hosts:");
+   for (i = 0; i < td->servers->items_len; ++i) {
+      sd = (mongoc_server_description_t *) mongoc_set_get_item (td->servers,
+                                                                (int) i);
+      MONGOC_DEBUG ("- %s", sd->host.host_and_port);
+   }
+}
+
 static void
 check_topology_description (mongoc_topology_description_t *td,
                             mongoc_host_list_t *hosts)
@@ -401,12 +482,16 @@ check_topology_description (mongoc_topology_description_t *td,
       BSON_ASSERT (mongoc_topology_description_add_server (
          td, host->host_and_port, NULL));
       if (server_count != td->servers->items_len) {
+         dump_topology_description (td);
+         dump_hosts (hosts);
          test_error ("topology description did not have host: %s",
                      host->host_and_port);
       }
    }
 
    if (nhosts != td->servers->items_len) {
+      dump_topology_description (td);
+      dump_hosts (hosts);
       test_error ("topology description had extra hosts");
    }
 }
@@ -522,6 +607,127 @@ test_small_initial_buffer (void *unused)
    _mongoc_host_list_destroy_all (rr_data.hosts);
 }
 
+bool
+_mock_resolver (const char *service,
+                mongoc_rr_type_t rr_type,
+                mongoc_rr_data_t *rr_data,
+                size_t initial_buffer_size,
+                bson_error_t *error)
+{
+   test_error ("Expected mock resolver to not be called");
+   return true;
+}
+
+static void
+_prose_loadbalanced_ping (mongoc_client_t *client)
+{
+   bson_error_t error;
+   bson_t *cmd = BCON_NEW ("ping", BCON_INT32 (1));
+
+   if (!mongoc_client_command_simple (
+          client, "admin", cmd, NULL, NULL, &error)) {
+      test_error ("ping failed: %s", error.message);
+   }
+
+   bson_destroy (cmd);
+}
+
+/*
+   Implements prose test 9 as described in the SRV polling test README:
+   Test that SRV polling is not done for load balanced clusters. Connect to
+   mongodb+srv://test3.test.build.10gen.cc/?loadBalanced=true, mock the addition
+   of the following DNS record, wait until 2*rescanSRVIntervalMS, and assert
+   that the final topology description only contains one server
+   (localhost.test.build.10gen.cc. at port 27017).
+   _mongodb._tcp.test3.test.build.10gen.cc. 86400 IN SRV 27018
+   localhost.test.build.10gen.cc.
+*/
+static void
+_prose_test_9 (bool pooled)
+{
+   mongoc_client_pool_t *pool;
+   mongoc_uri_t *uri;
+   mongoc_client_t *client;
+   mongoc_host_list_t *expected_hosts;
+   mongoc_topology_t *topology;
+#define RESCAN_INTERVAL_MS 500
+#ifdef MONGOC_ENABLE_SSL
+   mongoc_ssl_opt_t ssl_opts = *test_framework_get_ssl_opts ();
+
+   ssl_opts.allow_invalid_hostname = true;
+#endif
+
+
+   uri = mongoc_uri_new ("mongodb+srv://test3.test.build.10gen.cc");
+   mongoc_uri_set_option_as_bool (uri, MONGOC_URI_LOADBALANCED, true);
+   /* Single-threaded clients will only enter SRV polling during monitoring in
+    * mongoc_topology_scan_once. Reducing the heartbeatFrequencyMS will exercise
+    * the code path that would poll for SRV records. That should be bypassed
+    * because of the load balanced topology type. */
+   mongoc_uri_set_option_as_int32 (
+      uri, MONGOC_URI_HEARTBEATFREQUENCYMS, RESCAN_INTERVAL_MS);
+
+   if (pooled) {
+      pool = mongoc_client_pool_new (uri);
+      topology = _mongoc_client_pool_get_topology (pool);
+
+#ifdef MONGOC_ENABLE_SSL
+      mongoc_client_pool_set_ssl_opts (pool, &ssl_opts);
+#endif
+
+   } else {
+      client = mongoc_client_new_from_uri (uri);
+#ifdef MONGOC_ENABLE_SSL
+      mongoc_client_set_ssl_opts (client, &ssl_opts);
+#endif
+      topology = client->topology;
+   }
+
+   bson_mutex_lock (&topology->mutex);
+   _mongoc_topology_set_rr_resolver (topology, _mock_resolver);
+   _mongoc_topology_set_srv_polling_rescan_interval_ms (topology,
+                                                        RESCAN_INTERVAL_MS);
+   bson_mutex_unlock (&topology->mutex);
+
+   if (pooled) {
+      client = mongoc_client_pool_pop (pool);
+   }
+
+   _mongoc_usleep (2 * RESCAN_INTERVAL_MS * 1000);
+   /* For single-threaded, perform an operation since SRV polling occurs as a
+    * part of topology scanning. */
+   if (!pooled) {
+      _prose_loadbalanced_ping (client);
+   }
+
+   bson_mutex_lock (&topology->mutex);
+   expected_hosts = MAKE_HOSTS ("localhost.test.build.10gen.cc:27017");
+   check_topology_description (&client->topology->description, expected_hosts);
+   bson_mutex_unlock (&topology->mutex);
+
+   if (pooled) {
+      mongoc_client_pool_push (pool, client);
+      mongoc_client_pool_destroy (pool);
+   } else {
+      mongoc_client_destroy (client);
+   }
+
+   mongoc_uri_destroy (uri);
+   _mongoc_host_list_destroy_all (expected_hosts);
+}
+
+static void
+prose_test_9_single (void *unused)
+{
+   _prose_test_9 (false);
+}
+
+static void
+prose_test_9_pooled (void *unused)
+{
+   _prose_test_9 (true);
+}
+
 void
 test_dns_install (TestSuite *suite)
 {
@@ -532,12 +738,36 @@ test_dns_install (TestSuite *suite)
                       NULL,
                       NULL,
                       test_framework_skip_if_no_crypto);
-   TestSuite_AddFull (
-      suite, "/srv_polling/mocked", test_srv_polling_mocked, NULL, NULL, NULL);
+   TestSuite_AddFull (suite,
+                      "/initial_dns_seedlist_discovery/srv_polling/mocked",
+                      test_srv_polling_mocked,
+                      NULL,
+                      NULL,
+                      NULL);
    TestSuite_AddFull (suite,
                       "/initial_dns_seedlist_discovery/small_initial_buffer",
                       test_small_initial_buffer,
                       NULL,
                       NULL,
-                      test_dns_check);
+                      test_dns_check_replset);
+
+   /* TODO (CDRIVER-4045): remove /initial_dns_seedlist_discovery from the path
+    * of the SRV polling tests, since they are defined in the "Polling SRV
+    * Records for mongos Discovery" spec, not the "Initial DNS Seedlist
+    * Discovery" spec. */
+   TestSuite_AddFull (
+      suite,
+      "/initial_dns_seedlist_discovery/srv_polling/prose_test_9/single",
+      prose_test_9_single,
+      NULL,
+      NULL,
+      test_dns_check_loadbalanced);
+
+   TestSuite_AddFull (
+      suite,
+      "/initial_dns_seedlist_discovery/srv_polling/prose_test_9/pooled",
+      prose_test_9_pooled,
+      NULL,
+      NULL,
+      test_dns_check_loadbalanced);
 }
