@@ -31,7 +31,7 @@ test_split_insert (void)
    int i;
    bool r;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    BSON_ASSERT (client);
 
    collection = get_test_collection (client, "test_split_insert");
@@ -108,7 +108,7 @@ test_invalid_write_concern (void)
    bson_error_t error;
    bool r;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    BSON_ASSERT (client);
 
    collection = get_test_collection (client, "test_invalid_write_concern");
@@ -177,7 +177,7 @@ test_bypass_validation (void *context)
    int r;
    int i;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    BSON_ASSERT (client);
 
    dbname = gen_collection_name ("dbtest");
@@ -301,7 +301,7 @@ test_bypass_not_sent ()
    char *collname;
    char *dbname;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
 
    /* set up command monitoring for started commands */
    callbacks = mongoc_apm_callbacks_new ();
@@ -385,25 +385,25 @@ test_split_opquery_with_options (void)
    int n_docs;
 
    /* Use a reduced maxBsonObjectSize, and wire version for OP_QUERY */
-   const char *ismaster = "{'ok': 1.0,"
-                          " 'ismaster': true,"
-                          " 'minWireVersion': 0,"
-                          " 'maxWireVersion': 5,"
-                          " 'maxBsonObjectSize': 100}";
+   const char *hello = "{'ok': 1.0,"
+                       " 'isWritablePrimary': true,"
+                       " 'minWireVersion': 0,"
+                       " 'maxWireVersion': 5,"
+                       " 'maxBsonObjectSize': 100}";
 
    server = mock_server_new ();
-   mock_server_auto_ismaster (server, ismaster);
+   mock_server_auto_hello (server, hello);
    mock_server_run (server);
 
    /* Create an insert with two batches. Because of the reduced
-   * maxBsonObjectSize, each document must be less than 100 bytes.
-   * Because of the hardcoded allowance (see SERVER-10643), and our current
-   * incorrect batching logic (see CDRIVER-3310) the complete insert
-   * command can be can be 16k + 100 bytes.
-   * After CDRIVER-3310, update this test, since the allowance will not be
-   * taken into consideration for document batching.
-   * So create enough documents to fill up at least one batch.
-   */
+    * maxBsonObjectSize, each document must be less than 100 bytes.
+    * Because of the hardcoded allowance (see SERVER-10643), and our current
+    * incorrect batching logic (see CDRIVER-3310) the complete insert
+    * command can be can be 16k + 100 bytes.
+    * After CDRIVER-3310, update this test, since the allowance will not be
+    * taken into consideration for document batching.
+    * So create enough documents to fill up at least one batch.
+    */
    n_docs = ((BSON_OBJECT_ALLOWANCE) / tmp_bson ("{ '_id': 1 }")->len) +
             1; /* inexact, but errs towards more than enough documents. */
    docs = bson_malloc (sizeof (bson_t *) * n_docs);
@@ -411,7 +411,8 @@ test_split_opquery_with_options (void)
       docs[i] = BCON_NEW ("_id", BCON_INT64 (i));
    }
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    coll = mongoc_client_get_collection (client, "db", "coll");
 
    /* Add a write concern, to ensure that it is taken into account during
@@ -468,14 +469,14 @@ test_opmsg_disconnect_mid_batch_helper (int wire_version)
    int n_docs;
 
    /* Use a reduced maxBsonObjectSize, and wire version for OP_QUERY */
-   const char *ismaster = "{'ok': 1.0,"
-                          " 'ismaster': true,"
-                          " 'minWireVersion': 0,"
-                          " 'maxWireVersion': %d,"
-                          " 'maxBsonObjectSize': 100}";
+   const char *hello = "{'ok': 1.0,"
+                       " 'isWritablePrimary': true,"
+                       " 'minWireVersion': 0,"
+                       " 'maxWireVersion': %d,"
+                       " 'maxBsonObjectSize': 100}";
 
    server = mock_server_new ();
-   mock_server_auto_ismaster (server, ismaster, wire_version);
+   mock_server_auto_hello (server, hello, wire_version);
    mock_server_run (server);
 
    /* create enough documents for two batches. Note, because of our wonky
@@ -487,7 +488,8 @@ test_opmsg_disconnect_mid_batch_helper (int wire_version)
       docs[i] = BCON_NEW ("_id", BCON_INT64 (i));
    }
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    mongoc_client_set_error_api (client, MONGOC_ERROR_API_VERSION_2);
    coll = mongoc_client_get_collection (client, "db", "coll");
 
@@ -535,16 +537,17 @@ test_w0_legacy_insert_many (void)
 
    /* wire version will use OP_INSERT for w:0 insert many (since no OP_MSG) */
    server = mock_server_new ();
-   mock_server_auto_ismaster (server,
-                              "{'ismaster': true,"
-                              " 'maxWireVersion': 5}");
+   mock_server_auto_hello (server,
+                           "{'isWritablePrimary': true,"
+                           " 'maxWireVersion': 5}");
    mock_server_run (server);
 
    docs = bson_malloc (sizeof (bson_t *) * 2);
    docs[0] = BCON_NEW ("x", BCON_INT32 (1));
    docs[1] = BCON_NEW ("x", BCON_INT32 (2));
 
-   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
    coll = mongoc_client_get_collection (client, "db", "coll");
 
    /* Add unacknowldged write concern */
@@ -573,6 +576,126 @@ test_w0_legacy_insert_many (void)
    mock_server_destroy (server);
 }
 
+static void
+test_w0_legacy_update_one (void)
+{
+   mock_server_t *server;
+   mongoc_client_t *client;
+   mongoc_collection_t *coll;
+   bson_error_t error;
+   future_t *future;
+   request_t *request;
+   bson_t opts;
+   mongoc_write_concern_t *wc;
+
+   /* wire version will use OP_UPDATE for w:0 update (since no OP_MSG) */
+   server = mock_server_with_auto_hello (WIRE_VERSION_OP_MSG - 1);
+   mock_server_run (server);
+
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
+   coll = mongoc_client_get_collection (client, "db", "coll");
+
+   /* Add unacknowledged write concern */
+   bson_init (&opts);
+   wc = mongoc_write_concern_new ();
+   mongoc_write_concern_set_w (wc, 0);
+   mongoc_write_concern_append (wc, &opts);
+   mongoc_write_concern_destroy (wc);
+
+   future = future_collection_update_one (coll,
+                                          tmp_bson ("{'x':1}"),
+                                          tmp_bson ("{'$set': {'y':1}}"),
+                                          &opts,
+                                          NULL,
+                                          &error);
+
+   /* Mock server receives OP_UPDATE */
+   request = mock_server_receives_update (
+      server, "db.coll", 0, "{'x':1}", "{'$set': {'y':1}}");
+   BSON_ASSERT (request);
+
+   mock_server_replies_ok_and_destroys (request);
+   BSON_ASSERT (future_get_bool (future));
+
+   future_destroy (future);
+   bson_destroy (&opts);
+   mongoc_collection_destroy (coll);
+   mongoc_client_destroy (client);
+   mock_server_destroy (server);
+}
+
+static void
+test_w0_legacy_update_and_replace_validation (void)
+{
+   mock_server_t *server;
+   mongoc_client_t *client;
+   mongoc_collection_t *coll;
+   bson_error_t error;
+   bool r;
+   bson_t opts;
+   mongoc_write_concern_t *wc;
+
+   /* wire version will use OP_UPDATE for w:0 update (since no OP_MSG) */
+   server = mock_server_with_auto_hello (WIRE_VERSION_OP_MSG - 1);
+   mock_server_run (server);
+
+   client =
+      test_framework_client_new_from_uri (mock_server_get_uri (server), NULL);
+   coll = mongoc_client_get_collection (client, "db", "coll");
+
+   /* Add unacknowledged write concern */
+   bson_init (&opts);
+   wc = mongoc_write_concern_new ();
+   mongoc_write_concern_set_w (wc, 0);
+   mongoc_write_concern_append (wc, &opts);
+   mongoc_write_concern_destroy (wc);
+
+   /* replace_one prohibits top-level, dollar-prefixed keys */
+   memset (&error, 0, sizeof (bson_error_t));
+   r = mongoc_collection_replace_one (coll,
+                                      tmp_bson ("{'x':1}"),
+                                      tmp_bson ("{'$set': {'y':1}}"),
+                                      &opts,
+                                      NULL,
+                                      &error);
+
+   BSON_ASSERT (!r);
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_COMMAND,
+                          MONGOC_ERROR_COMMAND_INVALID_ARG,
+                          "Invalid key '$set': replace prohibits $ operators");
+
+   /* update_one requires top-level, dollar-prefixed keys */
+   memset (&error, 0, sizeof (bson_error_t));
+   r = mongoc_collection_update_one (
+      coll, tmp_bson ("{'x':1}"), tmp_bson ("{'y':1}"), &opts, NULL, &error);
+
+   BSON_ASSERT (!r);
+   ASSERT_ERROR_CONTAINS (
+      error,
+      MONGOC_ERROR_COMMAND,
+      MONGOC_ERROR_COMMAND_INVALID_ARG,
+      "Invalid key 'y': update only works with $ operators and pipelines");
+
+   /* update_many requires top-level, dollar-prefixed keys */
+   memset (&error, 0, sizeof (bson_error_t));
+   r = mongoc_collection_update_many (
+      coll, tmp_bson ("{'x':1}"), tmp_bson ("{'y':1}"), &opts, NULL, &error);
+
+   BSON_ASSERT (!r);
+   ASSERT_ERROR_CONTAINS (
+      error,
+      MONGOC_ERROR_COMMAND,
+      MONGOC_ERROR_COMMAND_INVALID_ARG,
+      "Invalid key 'y': update only works with $ operators and pipelines");
+
+   bson_destroy (&opts);
+   mongoc_collection_destroy (coll);
+   mongoc_client_destroy (client);
+   mock_server_destroy (server);
+}
+
 static void
 _configure_failpoint (mongoc_client_t *client,
                       const char *mode,
@@ -602,7 +725,7 @@ _test_invalid_wc_server_error (void *unused)
    bson_t reply;
    bson_error_t error;
 
-   client = test_framework_client_new ();
+   client = test_framework_new_default_client ();
    mongoc_client_set_error_api (client, MONGOC_ERROR_API_VERSION_2);
    coll = get_test_collection (client, "server_wc_error");
 
@@ -648,10 +771,18 @@ test_write_command_install (TestSuite *suite)
                                 test_opmsg_disconnect_mid_batch);
    TestSuite_AddMockServerTest (
       suite, "/WriteCommand/w0_legacy_insert_many", test_w0_legacy_insert_many);
+   TestSuite_AddMockServerTest (
+      suite, "/WriteCommand/w0_legacy_update_one", test_w0_legacy_update_one);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/WriteCommand/w0_legacy_update_and_replace_validation",
+      test_w0_legacy_update_and_replace_validation);
    TestSuite_AddFull (suite,
                       "/WriteCommand/invalid_wc_server_error",
                       _test_invalid_wc_server_error,
                       NULL,
                       NULL,
-                      test_framework_skip_if_no_failpoint);
+                      test_framework_skip_if_no_failpoint,
+                      /* Skipping on sharded clusters; see SERVER-57772 */
+                      test_framework_skip_if_mongos);
 }
