@@ -60,18 +60,25 @@ skipped_unified_test_t SKIPPED_TESTS[] = {
    {"unacknowledged-updateOne-hint-clientError", "Unacknowledged updateOne with hint document fails with client-side error"},
    /* CDRIVER-4001, DRIVERS-1781, and DRIVERS-1448: 5.0 cursor behavior */
    {"poc-command-monitoring", "A successful find event with a getmore and the server kills the cursor"},
+   /* CDRIVER-3867: drivers atlas testing (schema version 1.2) */
+   {"entity-client-storeEventsAsEntities", SKIP_ALL_TESTS},
+   /* libmongoc does not have a distinct helper, so skip snapshot tests testing particular distinct functionality */
+   {"snapshot-sessions", "Distinct operation with snapshot"},
+   {"snapshot-sessions", "Mixed operation with snapshot"},
    /* CDRIVER-3886: serverless testing (schema version 1.4) */
    {"poc-crud", SKIP_ALL_TESTS},
    {"db-aggregate", SKIP_ALL_TESTS},
    {"mongos-unpin", SKIP_ALL_TESTS},
-   /* CDRIVER-3883: load balancer support (schema version 1.3) */
+   /* CDRIVER-2871: CMAP is not implemented */
    {"assertNumberConnectionsCheckedOut", SKIP_ALL_TESTS},
    {"entity-client-cmap-events", SKIP_ALL_TESTS},
-   {"entity-find-cursor", SKIP_ALL_TESTS},
    {"expectedEventsForClient-eventType", SKIP_ALL_TESTS},
-   {"ignoreResultAndError", SKIP_ALL_TESTS},
-   /* CDRIVER-3867: comprehensive Atlas testing (schema version 1.2) */
-   {"entity-client-storeEventsAsEntities", SKIP_ALL_TESTS},
+   /* CDRIVER-4115: listCollections does not support batchSize. */
+   {"cursors are correctly pinned to connections for load-balanced clusters", "listCollections pins the cursor to a connection"},
+   /* CDRIVER-4116: listIndexes does not support batchSize. */
+   {"cursors are correctly pinned to connections for load-balanced clusters", "listIndexes pins the cursor to a connection"},
+   /* libmongoc does not pin connections to cursors. It cannot force an error from waitQueueTimeoutMS by creating cursors in load balanced mode. */
+   {"wait queue timeout errors include details about checked out connections", SKIP_ALL_TESTS},
    {0},
 };
 /* clang-format on */
@@ -372,10 +379,13 @@ test_runner_new (void)
    callbacks = mongoc_apm_callbacks_new ();
    mongoc_apm_set_topology_changed_cb (callbacks, on_topology_changed);
    uri = test_framework_get_uri ();
-   /* Always use multiple mongos's if speaking to a mongos.
-    * Some test operations require communicating with all known mongos */
-   if (!test_framework_uri_apply_multi_mongos (uri, true, &error)) {
-      test_error ("error applying multiple mongos: %s", error.message);
+   /* In load balanced mode, the internal client must use the SINGLE_LB_MONGOS_URI. */
+   if (!test_framework_is_loadbalanced ()) {
+      /* Always use multiple mongoses if speaking to a mongos.
+       * Some test operations require communicating with all known mongos */
+      if (!test_framework_uri_apply_multi_mongos (uri, true, &error)) {
+         test_error ("error applying multiple mongos: %s", error.message);
+      }
    }
    test_runner->internal_client =
       test_framework_client_new_from_uri (uri, NULL);
@@ -390,6 +400,8 @@ test_runner_new (void)
       get_topology_type (test_runner->internal_client);
    server_semver (test_runner->internal_client, &test_runner->server_version);
 
+   test_runner->is_serverless = test_framework_is_serverless ();
+
    /* Terminate any possible open transactions. */
    if (!test_runner_terminate_open_transactions (test_runner, &error)) {
       test_error ("error terminating transactions: %s", error.message);
@@ -543,6 +555,10 @@ get_topology_type (mongoc_client_t *client)
    bson_error_t error;
    const char *topology_type = "single";
 
+   if (test_framework_is_loadbalanced ()) {
+      return "load-balanced";
+   }
+
    ret = mongoc_client_command_simple (
       client, "admin", tmp_bson ("{'hello': 1}"), NULL, &reply, &error);
    if (!ret) {
@@ -602,7 +618,7 @@ get_topology_type (mongoc_client_t *client)
 static void
 check_schema_version (test_file_t *test_file)
 {
-   const char *supported_version_strs[] = {"1.1", "1.5"};
+   const char *supported_version_strs[] = {"1.5"};
    int i;
 
    for (i = 0; i < sizeof (supported_version_strs) /
@@ -716,6 +732,33 @@ check_run_on_requirement (test_runner_t *test_runner,
          continue;
       }
 
+      if (0 == strcmp (key, "serverless")) {
+         const char *serverless_mode = bson_iter_utf8 (&req_iter, NULL);
+
+         if (0 == strcmp (serverless_mode, "allow")) {
+            continue;
+         } else if (0 == strcmp (serverless_mode, "require")) {
+            if (!test_runner->is_serverless) {
+               *fail_reason =
+                  bson_strdup_printf ("Not running in serverless mode");
+               return false;
+            }
+
+            continue;
+         } else if (0 == strcmp (serverless_mode, "forbid")) {
+            if (test_runner->is_serverless) {
+               *fail_reason = bson_strdup_printf ("Running in serverless mode");
+               return false;
+            }
+
+            continue;
+         } else {
+            test_error ("Unexpected serverless mode: %s", serverless_mode);
+         }
+
+         continue;
+      }
+
       if (0 == strcmp (key, "auth")) {
          bool auth_requirement = bson_iter_bool (&req_iter);
 
@@ -965,6 +1008,7 @@ test_check_event (test_t *test,
    char *expected_command_name = NULL;
    char *expected_database_name = NULL;
    bson_t *expected_reply = NULL;
+   bool *expected_has_service_id = NULL;
 
    if (bson_count_keys (expected) != 1) {
       test_set_error (error,
@@ -997,6 +1041,7 @@ test_check_event (test_t *test,
    bson_parser_utf8_optional (bp, "commandName", &expected_command_name);
    bson_parser_utf8_optional (bp, "databaseName", &expected_database_name);
    bson_parser_doc_optional (bp, "reply", &expected_reply);
+   bson_parser_bool_optional (bp, "hasServiceId", &expected_has_service_id);
    if (!bson_parser_parse (bp, &expected_bson, error)) {
       goto done;
    }
@@ -1054,6 +1099,22 @@ test_check_event (test_t *test,
       bson_val_destroy (actual_val);
    }
 
+   if (expected_has_service_id) {
+      char oid_str[25] = {0};
+      bool has_service_id = false;
+
+      bson_oid_to_string (&actual->service_id, oid_str);
+      has_service_id = 0 != bson_oid_compare (&actual->service_id, &kZeroServiceId);
+
+      if (*expected_has_service_id && !has_service_id) {
+         test_error ("expected serviceId, but got none");
+      }
+      
+      if (!*expected_has_service_id && has_service_id) {
+         test_error ("expected no serviceId, but got %s", oid_str);
+      }
+   }
+
    ret = true;
 done:
    bson_parser_destroy_with_parsed_fields (bp);
@@ -1074,14 +1135,28 @@ test_check_expected_events_for_client (test_t *test,
    event_t *eiter = NULL;
    uint32_t expected_num_events;
    uint32_t actual_num_events = 0;
+   char *event_type = NULL;
 
    bp = bson_parser_new ();
    bson_parser_utf8 (bp, "client", &client_id);
    bson_parser_array (bp, "events", &expected_events);
+   bson_parser_utf8_optional (bp, "eventType", &event_type);
    if (!bson_parser_parse (bp, expected_events_for_client, error)) {
       goto done;
    }
 
+   if (event_type) {
+      if (0 == strcmp (event_type, "cmap")) {
+         /* TODO: (CDRIVER-3525) Explicitly ignore cmap events until CMAP is
+          * supported. */
+         ret = true;
+         goto done;
+      } else if (0 != strcmp (event_type, "command")) {
+         test_set_error (error, "unexpected event type: %s", event_type);
+         goto done;
+      }
+   }
+
    entity = entity_map_get (test->entity_map, client_id, error);
    if (0 != strcmp (entity->type, "client")) {
       test_set_error (error,
@@ -1599,10 +1674,15 @@ test_install_unified (TestSuite *suite)
 {
    run_unified_tests (suite, JSON_DIR "/unified");
 
-   /* crud/unified spec tests also use the unified format. */
    run_unified_tests (suite, JSON_DIR "/crud/unified");
 
    run_unified_tests (suite, JSON_DIR "/transactions/unified");
 
    run_unified_tests (suite, JSON_DIR "/collection-management");
+
+   run_unified_tests (suite, JSON_DIR "/sessions/unified");
+
+   run_unified_tests (suite, JSON_DIR "/change_streams/unified");
+
+   run_unified_tests (suite, JSON_DIR "/load_balancers");
 }
